@@ -1,5 +1,5 @@
 parseDVH <- function(x, type=c("Eclipse", "Cadplan", "Masterplan",
-                               "Pinnacle", "Monaco", "HiArt")) {
+                               "Pinnacle", "Monaco", "HiArt", "RayStation")) {
     type <- match.arg(type)
 
     ## name them using patient IDs
@@ -7,6 +7,10 @@ parseDVH <- function(x, type=c("Eclipse", "Cadplan", "Masterplan",
         if(type == "Monaco") {
             header <- unlist(strsplit(txt[1], " [|] "))
             trimWS(sub("^Patient ID: .+[~](.+)$", "\\1", header[1]))
+        } else if(type == "RayStation") {
+            IDline <- txt[grep("^(#PatientId):.+", txt)]
+            IDres  <- sub("^.+?:[[:blank:]]*([[:alnum:][:punct:][:blank:]]+$)", "\\1", IDline, perl=TRUE)
+            collWS(trimWS(IDres, side="both"))
         } else if(type == "HiArt") {
             gsub("[^a-z0-9]", "\\1", tempfile(pattern="", tmpdir=""))
         } else if(type != "Pinnacle") {
