@@ -19,7 +19,8 @@ shinyServer(function(input, output) {
             } else if(input$DVHin == '2') {
                 if(!is.null(input$DVHupload)) {
                     types <- c('1'="Eclipse", '2'="Cadplan", '3'="Masterplan",
-                               '4'="Pinnacle", '5'="Monaco", '6'="HiArt")
+                               '4'="Pinnacle", '5'="Monaco", '6'="HiArt",
+                               '7'="RayStation")
                     plans <- c('1'="none", '2'="doseRx")
                     argL  <- if(("DVHadd" %in% input$DVHreadOpts) && file.exists("DVHprev.rds")) {
                         setwd(DVHdir)
